//  GAMBIT: Global and Modular BSM Inference Tool
//  *********************************************
///  \file
///
///  Simple base class for interfacing user-defined
///  functions to the UserFuncPrior class
///
///  *********************************************
///
///  Authors (add name and date if you modify):
///   
///  \author Ben Farmer
///          (b.farmer@imperial.ac.uk))
///  \date 2018 Dec
///
///  *********************************************

#ifndef __USER_FUNC_HPP__
#define __USER_FUNC_HPP__ 

#include <vector>
#include <string>

namespace Gambit
{
    namespace Priors
    { 
        /// Base wrapper class for user-supplied function
        /// Users can provide an object from a custom derived 
        /// class instead, if their function has special 
        /// wrapping requirements. 
        class UserFunc
        {
            private:
                // Parameters that this function is supposed to act on
                std::vector<std::string> my_parameters;
            public:
                //UserFunc constructors
                UserFunc() : my_parameters() {}
                UserFunc(const std::vector<std::string>& pars)
                    : my_parameters(pars)
                {}

                // transform function interface
                virtual std::vector<double> transform(const std::vector<double>&) const = 0;

                // Retrieve parameters upon which this function acts
                std::vector<std::string> getParameters() { return my_parameters; }
        };
    }
}

#endif
