//  GAMBIT: Global and Modular BSM Inference Tool
//  *********************************************
///  \file
///
///  Prior object to wrap a user-supplied
///  inverse transform function
///
///  *********************************************
///
///  Authors (add name and date if you modify):
///   
///  \author Ben Farmer
///          (b.farmer@imperial.ac.uk))
///  \date 2018 Dec
///
///  *********************************************

#ifndef PRIOR_USER_FUNC_HPP
#define PRIOR_USER_FUNC_HPP

#include <vector>
#include "gambit/ScannerBit/priors.hpp"
#include "gambit/Utils/yaml_options.hpp"

namespace Gambit
{
    namespace Priors
    {
        class UserFunc; // Forward declaration

        // User-supplied function prior. Takes no arguments; it is assumed that the user
        // has constructed it exactly as they want it.
        class UserFuncPrior : public BasePrior
        {
        private:
            UserFunc const * my_func;
            bool user_func_set;

        public: 
            // Constructor defined in gaussian.cpp
            UserFuncPrior(const std::vector<std::string>& params, const Options& options);
           
            // Function to set the user-supplied transform function
            void set_transform_func(UserFunc const * const user_func);

            // Transformation from unit cube to the model parameter space 
            void transform(const std::vector <double> &unitpars, std::unordered_map <std::string, double> &outputMap) const;
 
            // I think this is the prior density. We don't actually use this so far, I think.           
            double operator()(const std::vector<double> &vec) const;

        };

        LOAD_PRIOR(user, UserFuncPrior)
    }
}

#endif
